///
/// The base game object
///

#ifndef __GAME_OBJECT_H__
#define __GAME_OBJECT_H__

#include "Position.h"		///< Objects need a location.
#include "Box.h"		///< Objects have a bounding box.
#include "Event.h"		///< Objects can handle events.
#include "Sprite.h"		///< Objects (often) have sprites.

class GameObject {

 private:
  int id;			///< Globally unique id of object

 protected:
  string type;		        ///< User-defined identification.
  Position pos;			///< Position in the game world.
  Box box;			///< Bounding box for collisions.
  bool is_solid;		///< True if object is solid.
  int altitude;			///< -MAX to MAX supported (lower drawn first).
  float x_velocity;		///< Horizontal speed in spaces per game step.
  float x_velocity_countdown;	///< Countdown to horizontal movement.
  float y_velocity;		///< Veritical speed in spaces per game step.
  float y_velocity_countdown;	///< Countdown to vertical movement.
  bool is_visible;		///< If true, object gets drawn.
  Sprite *p_sprite;		///< The sprite associated with this object.
  bool sprite_center;	        ///< True if sprite is centered on object.
  int sprite_index;		///< Current index frame for sprite.
  int sprite_slowdown;		///< Slowdown rate (1 = no slowdown, 0 = stop).
  int sprite_slowdown_count;    ///< Slowdown counter.
  
 public:
  GameObject();
  virtual ~GameObject();
  void setId(int new_id);
  int getId();
  void setType(string new_type);
  string getType();
  void setSolid(bool solid);
  bool isSolid();
  void setVisible(bool visible);
  bool isVisible();
  void setPosition(Position new_pos);
  Position getPosition();
  void setBox(Box new_box);
  Box getBox();
  void setAltitude(int new_altitude);
  int getAltitude();

  void setXVelocity(float new_x_velocity);
  void setYVelocity(float new_y_velocity);
  float getXVelocity();
  float getYVelocity();

  /// Perform 1 step of velocity in horizontal direction.
  /// Return horizontal distance moved this step.
  int getXVelocityStep();

  /// Perform 1 step of velocity in vertical direction.
  /// Return vertical distance moved this step.
  int getYVelocityStep();

  /// Handle event (default is to ignore everything).
  /// Return 0 if ignored, else 1
  virtual int eventHandler(Event *p_event);

  /// Draw single sprite frame.
  /// Drawing accounts for: centering, slowdown,
  /// advancing (as appropriate for each).
  virtual void draw();

  /// Set object sprite to new one.
  /// If set_box is true, set bounding box to size of sprite (default is true).
  void setSprite(Sprite *p_new_sprite);
  void setSprite(Sprite *p_new_sprite, bool set_box);
  Sprite *getSprite();

  /// Indicates if sprite is centered at object Position (pos).
  bool isCentered();

  /// Indicate sprite is to centered at object Position (pos).
  void setCentered(bool centered);

  /// Get the index of current Sprite frame to be displayed.
  int getSpriteIndex();

  /// Set the index of current Sprite frame to be displayed.
  void setSpriteIndex(int new_sprite_index);

  /// Slows down sprite animations.
  /// new_sprite_slowdown is in multiples of WorldManager frame time.
  void setSpriteSlowdown(int new_sprite_slowdown);
  int getSpriteSlowdown();
  void setSpriteSlowdownCount(int new_sprite_slowdown_count);
  int getSpriteSlowdownCount();
};
 
#endif // __GAME_OBJECT_H__
