///
/// The game world manager
///

#ifndef __WORLD_MANAGER_H__
#define __WORLD_MANAGER_H__

#include "Manager.h"
#include "GameObjectList.h"
#include "Position.h"
#include "Box.h"

#define MAX_ALTITUDE 2		       ///< Drawing order (-MAX to MAX)

class WorldManager : public Manager {

 private:
  int obj_id;			       ///< Next object id to assign.
  WorldManager (WorldManager const&);  ///< Don't allow copy.
  void operator=(WorldManager const&); ///< Don't allow assignment.
  WorldManager();		       ///< Private since a singleton.

 protected:
  GameObjectList objects;       ///< List of all game objects.
  GameObjectList deletions;     ///< List of all game objects to delete.
  GameObject *p_view_following; ///< Game object view is following.
  Box boundary;			///< World boundaries.
  Box view;			///< Player window view.
  bool valueInRange(int value,	       
		    int min, int max); ///< True if min <= value <= max

 public:
  ~WorldManager();

  /// Get the one and only instance of the WorldManager.
  static WorldManager &getInstance();

  /// \brief Startup the game world (initialize everthing to empty).
  /// Return 0.
  int startUp();

  /// Shutdown the game world.
  void shutDown();

  /// Indicate object is to be deleted at end of current game loop.
  /// Return 0 if ok, else -1.
  int markForDelete(GameObject *p_go);

  /// Return a list of all game world objects (obj).
  /// Return  NULL if list is empty.
  GameObjectList getAllObjects(void);

  /// Add object to game world (obj). 
  /// Return 0 if ok, else -1.
  int insertObject(GameObject *p_go);

  /// Remove object from game world (obj). 
  /// Return 0 if ok, else -1.
  int removeObject(GameObject *p_go);

  /// Return pointer to object with indicated id.
  /// Return NULL if object not found.
  GameObject *findObject(int id);

  /// Update world.
  /// Send "step" event to all interested objects.
  /// Update positions of objects based on their velocities.
  /// Lastly, delete those marked for deletion.
  void update();

  /// Draw, sending draw event to all objects (in visible viewport).
  /// Draw bottom up, from -MAX_ALTITUDE to MAX_ALTITUDE
  void draw();

  /// Check if boxes intersect. 
  /// Return true if boxes intersect, else false.
  bool boxesIntersect(Box box1, Box box2);

  /// Move object. 
  /// If there is a collision, don't move object.
  /// If move, then adjust camera if following this object.
  /// Return 0 if move ok, else -1 if collision with solid.
  /// If p_go is a solid game object, allow move and return 0.
  int moveObject(GameObject *p_go, Position where);

  /// Check if object has collision w/any other solid object at position where.
  /// Return pointer to first object collided with, else return NULL.
  /// Does not consider if p_go is solid or not.
  GameObject *isCollision(GameObject *p_go, Position where);

  /// Get game world boundary.
  Box getBoundary();

  /// Set game world boundary.
  void setBoundary(Box new_boundary);

  /// Get camera viewport for game world.
  Box getView();

  /// Set camera viewport for game world.
  void setView(Box new_view);

  /// Set camera viewport to center camera on object.
  /// If p_new_view_following not legit, return -1 else return 0.
  /// Set to NULL to stop following.
  int setViewFollowing(GameObject *p_new_view_following);

  /// Set camera viewport to center on position view_pos.
  /// Viewport edge will not go beyond world boundary.
  void setViewPosition(Position view_pos);
};

#endif // __WORLD_MANAGER_H__
